#include <netcdf.h>
#include "geminc.h"
#include "gemprm.h"
#include "nc2grid.h"

void nc_getgridnav (int ncid, ncdim_t *dims, int ndims, ncgatt_t *gatts, 
                    int ngatts, ncvar_t var, int nvars, char *cproj, int *kx, 
                    int *ky, float *gdsarr, float *corners, float *navblk, 
                    int *ntimes, int *ncfiletype, int *iret)

/************************************************************************
 * Search the netCDF file and return GRID NAVIGATION information	*
 *									*
 * Input parameters:                                                    *
 * 	ncid		int		netCDF file number		*
 *      dims            ncdim_t         file diminsions definitions	*
 *      ndims		int		number variables in file	*
 *	gatts		ncgatt_t	global attributes		*
 *      ngatts          int		number of global attributes     *
 *      var             ncvar_t         varaible definitions		*
 *      nvars           int             number of variables in NC file	*
 *									*
 * Output parameters:							*
 *	*cproj		char		Grid Projection			*
 *	*kx		int		number of columns		*
 *	*ky		int		number of rows			*
 *	*gdsarr		float		GDS information array		*
 *                     gdsarr[0] = Grid Type                            *
 *                     gdsarr[1] = Num of points in X direction         *
 *                     gdsarr[2] = Num of points in Y direction         *
 *                     gdsarr[3] = Lat of lower left grid point         *
 *                     gdsarr[4] = Lon of lower left grid point         *
 *                     gdsarr[5] = Lat of upper right grid point        *
 *                     gdsarr[6] = Lon of upper right grid point        *
 *                     gdsarr[7] = angle1;                              *
 *                     gdsarr[8] = angle2;                              *
 *                     gdsarr[9] = angle3;                              *
 *      *corners        float           Grid corners                    *
 *      *navblk         float           Grid navigation block           *
 *      ntimes          int             Number of times in file		*
 *	*ncfiletype     int		netcdf file type		*
 *					    1 - GFE netCDF		*
 *					    2 - AWIPS D2D netCDF	*
 *					    3 - CDC reanalysis netCDF	*
 *					    4 - NARR reanalysis netCDF	*
 *					    5 - NAO netCDF              *
 *					    6 - HRAP			*
 *					    7 - Auto Front file         *
 *	iret		int		return value			*
 *									*
 ************************************************************************/
{
	int dimid, gattid, varid, nc_status, angflg, valid, ier;
	int latlongrid,projfound, k, ix, iy;
	int gfeflag=0;
	float *var_val;
        double *var_val_double;
	float domainExt[2], domainOrig[2];

	*iret = 0;
	*ncfiletype = 0;

	// Initialize domain Origin and domain extent (Needed for GFE netCDF)
	domainExt[0] = -9999.0;
	domainExt[1] = -9999.0;
	domainOrig[0] = -9999.0;
	domainOrig[1] = -9999.0;
        /* 
	 * Find the Grids x,y in dimensions.
	 */
	latlongrid = -1;
        for (dimid = 0; dimid < ndims; dimid++ ) {
            if (strcmp(dims[dimid].name, "x") == 0) {
		gdsarr[1] = dims[dimid].size;
                latlongrid = 0;
		*ncfiletype = 2;
                break;
            } else if (strcmp(dims[dimid].name, "hrapx") == 0) {
                gdsarr[1] = dims[dimid].size;
                latlongrid = 0;
                *ncfiletype = 6;
                break;
            } else if (strcmp(dims[dimid].name, "lon") == 0 ) {
		gdsarr[1] = dims[dimid].size;
                //printf("*** %f\n", gdsarr[1]);
                latlongrid = 1;
		*ncfiletype = 3;
		
		/* Search for NAO in global attributes */
		int naofound = 0;
		for (gattid = 0; gattid < ngatts; gattid++) {
		    if (naofound == 0 && strcmp(gatts[gattid].name, "institution") == 0) {
		        if (strcmp(gatts[gattid].string, "Naval Oceanographic Office") == 0 ) {
			    *ncfiletype = 5;
			}
		    }
		}
                break;
            } else if (strcmp(dims[dimid].name, "longitude") == 0 ) {
                gdsarr[1] = dims[dimid].size;
                latlongrid = 1;
                *ncfiletype = 3;
            } else {
		/* Grid Size not found in conventional dimensions section */
		//*iret = *iret - 1;
		gdsarr[1] = -1;
            }
        }

        for (dimid = 0; dimid < ndims; dimid++ ) {
            if (strcmp(dims[dimid].name, "y") == 0 ) {
		gdsarr[2] = dims[dimid].size;
                break;
            } else if (strcmp(dims[dimid].name, "hrapy") == 0 ) {
                gdsarr[2] = dims[dimid].size;
                break;
            } else if (strcmp(dims[dimid].name, "lat") == 0 ) {
		gdsarr[2] = dims[dimid].size;
                break;
            } else {
		/* Grid Size not found in conventional dimensions section */
                //*iret = *iret - 1;
		gdsarr[2] = -1;
            }
        }

	*kx = rint ( gdsarr [1] );
	*ky = rint ( gdsarr [2] );

        for (dimid = 0; dimid < ndims; dimid++ ) {
            if (strcmp(dims[dimid].name, "n_valtimes") == 0 ) {
                *ntimes = dims[dimid].size;
                break;
            } else if (strcmp(dims[dimid].name, "times") == 0 ||
		       strcmp(dims[dimid].name, "time") == 0 ) {
                *ntimes = dims[dimid].size;
                break;
            } else {
		if ( *ncfiletype == 6 ) {
		    *ntimes = 1;
		} else {
		/* unable to find the number of times in dimension section */
		//if (iret != 0) {
		//    *iret = -4;
		//}
	 	    *ntimes = -1;
		}
	    }
        }
	/*
	 * Find Grid Projection
	 */
	projfound = 0;
        /* Search Global Atributes for grid projection name First */
        for (gattid = 0; gattid < ngatts; gattid++) {
            if (projfound == 0 && strcmp(gatts[gattid].name, "projName") == 0) {
                if (strcmp(gatts[gattid].string, "STEREOGRAPHIC") == 0 || strcmp(gatts[gattid].string, "POLAR_STEREOGRAPHIC") == 0) {
                    strcpy(cproj, "STR");
		    gdsarr[0] = (float) 5.0;
                    projfound = 1;
                } else if (strcmp(gatts[gattid].string, "LAMBERT_CONFORMAL") == 0 ) {
                    strcpy(cproj, "LCC");
		    gdsarr[0] = (float) 3.0;
                    projfound = 1;
                } else if (strcmp(gatts[gattid].string, "CYLINDRICAL_EQUIDISTANT") == 0 ) {
                    strcpy(cproj, "CED");
		    gdsarr[0] = (float) 0.0;
                    projfound = 1;
                } else if (strcmp(gatts[gattid].string, "MERCATOR") == 0 ) {
                    strcpy(cproj, "MER");
                    gdsarr[0] = (float) 1.0;
                    projfound = 1;
		} 
            }

        }
	/*
	 * Projection Name not found in Global Attributes but grid dimensions are
	 * defines as lat lon. Grid is a latlon grid
	 */
        if (projfound == 0) {
	    if (latlongrid == 1) {
		/* 
		 * Assume CDC 2.5x2.5 degree Lat/Lon reanalysis netCDF
		 */
                strcpy(cproj, "CED");
	        gdsarr[0] = (float) 0.0;
            } else {
		// if HRAP grid, get corners
		if ( *ncfiletype == 6 ) {
			projfound = 1;
		    strcpy(cproj, "STR");
		    gdsarr[0] = (float) 5.0;
		    for (varid = 0; varid < nvars; varid++ ) {
                        nc_status = nc_inq_varndims(ncid, varid, &var.ndims);
                        var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
                        nc_status = nc_inq_var (ncid, varid, var.name, &var.type, 0, var.dims, &var.natts);
                        if (var.type = NC_FLOAT) {
                            // get variable dimensions
                            int k;
                            int array_size = 1;
                            for (k = 0; k < var.ndims; k++) {
                                array_size = array_size * dims[var.dims[k]].size;
                            }
                            var_val = (float *) malloc(array_size * sizeof(float));
                            nc_status = nc_get_var_float(ncid, varid, var_val);
                            if (strcmp(var.name, "lat") == 0) {
                                gdsarr[3] = var_val[0];
                                gdsarr[5] = var_val[2];
                            } else if (strcmp(var.name, "lon") == 0) {
                                gdsarr[4] = -1*var_val[0];
			        gdsarr[6] = -1*var_val[2];
			    } else if (strcmp(var.name, "true_lon") == 0) {
				gdsarr[8] = -1*var_val[0];
			    } else if (strcmp(var.name, "true_lat") == 0) {
                                //gdsarr[7] = var_val[0];
				gdsarr[7] = 90;
				gdsarr[9] = 0;
                            }
			}
		    }
		} else {

		/*
		 * Assume NARR netCDF file since projection not found and not a lat/lon grid
		 */
		for (varid = 0; varid < nvars; varid++ ) {
                    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);
                    var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
                    nc_status = nc_inq_var (ncid, varid, var.name, &var.type, 0, var.dims, &var.natts);
		    /* a variable with the name of the grid projection should be in the file */
                    if (strcmp(var.name, "Lambert_Conformal") == 0 ) {
			strcpy(cproj, "LCC");
                        gdsarr[0] = (float) 3.0;
			*ncfiletype = 4;
			projfound = 1;
			/* The rest of the grid information is from global attributes */
			for (gattid = 0; gattid < ngatts; gattid++) {
                	    if (strcmp(gatts[gattid].name, "latcorners" ) == 0) {
				if (gatts[gattid].type == NC_FLOAT) {
				    gdsarr[3] = gatts[gattid].fvals[0];
				    gdsarr[5] = gatts[gattid].fvals[2];
				} else if (gatts[gattid].type == NC_DOUBLE) { 
                                    gdsarr[3] = (float)gatts[gattid].dvals[0];
                                    gdsarr[5] = (float)gatts[gattid].dvals[2];
				}
			    } 
			    if (strcmp(gatts[gattid].name, "loncorners" ) == 0) {
				if (gatts[gattid].type == NC_FLOAT) {
                                    gdsarr[4] = gatts[gattid].fvals[0];
                                    gdsarr[6] = gatts[gattid].fvals[2];
				} else if (gatts[gattid].type == NC_DOUBLE) {
				    gdsarr[4] = (float)gatts[gattid].dvals[0];
                                    gdsarr[6] = (float)gatts[gattid].dvals[2];
				}
                            }
			    if (strcmp(gatts[gattid].name, "standardpar1" ) == 0) {
				if (gatts[gattid].type == NC_FLOAT) {
				    gdsarr[7] = *gatts[gattid].fvals;
				} else if (gatts[gattid].type == NC_DOUBLE) {
			  	    gdsarr[7] = (float)*gatts[gattid].dvals;
				}
			    }
			    if (strcmp(gatts[gattid].name, "standardpar2" ) == 0) { 
				if (gatts[gattid].type == NC_FLOAT) {
				    gdsarr[9] = *gatts[gattid].fvals;
				} else if (gatts[gattid].type == NC_DOUBLE) {
				    gdsarr[9] = (float)*gatts[gattid].dvals;
				}
                            }
			    if (strcmp(gatts[gattid].name, "centerlon" ) == 0) {
				if (gatts[gattid].type == NC_FLOAT) {
				    gdsarr[8] = *gatts[gattid].fvals;
				} else if (gatts[gattid].type == NC_DOUBLE) {
				    gdsarr[8] = (float)*gatts[gattid].dvals;
				}
                            }
			}
			break;
		    }
		}
		}
	    }
	}
	/*
	 * Find Grid corners and projection angles for AWIPS D2D Grid
	 */
        if ( projfound == 1 ) {
            for (gattid = 0; gattid < ngatts; gattid++) {
                if (strcmp(gatts[gattid].name, "lat00" ) == 0) {
                    gdsarr[3] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "lon00" ) == 0) {
                    gdsarr[4] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "latNxNy" ) == 0) {
                    gdsarr[5] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "lonNxNy" ) == 0) {
                    gdsarr[6] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "centralLat" ) == 0) {
                    gdsarr[7] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "centralLon" ) == 0) {
                    gdsarr[8] = *gatts[gattid].fvals;
                }
                if (strcmp(gatts[gattid].name, "rotation" ) == 0) {
                    gdsarr[9] = *gatts[gattid].fvals;
                }
            }
        } else {
	    /*
	     * Find Lat/Lon info from Lat & Lon variables for a lat/lon reanalysis 
	     */
            if ( latlongrid == 1 ) {
                /* Assuming lat/Lon grid since dimensions are Lat, Lon
                 * Loop thru variables looking for lat, lon */
                for (varid = 0; varid < nvars; varid++ ) {
                    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);
                    var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
                    nc_status = nc_inq_var (ncid, varid, var.name, &var.type, 0, var.dims, &var.natts);
                    if (strcmp(var.name, "lat") == 0 || strcmp(var.name, "lon") == 0 || strcmp(var.name, "latitude") == 0 || strcmp(var.name, "longitude") == 0 ) {
                        if (var.type == NC_FLOAT) {
                            // get variable dimensions
                            int k;
                            int array_size = 1;
                            for (k = 0; k < var.ndims; k++) {
                                array_size = array_size * dims[var.dims[k]].size;
                            }
                            var_val = (float *) malloc(array_size * sizeof(float));
                            nc_status = nc_get_var_float(ncid, varid, var_val);
                            
                           if (strcmp(var.name, "lat") == 0 || strcmp(var.name, "latitude") == 0 ) {
                                gdsarr[3] = var_val[0];
                                gdsarr[5] = var_val[array_size-1];
				if (gdsarr[3] > gdsarr[5]) {
				    gdsarr[5] = var_val[0];
				    gdsarr[3] = var_val[array_size-1];
				    *ncfiletype = 3;
				}
				gdsarr[2] = (float)array_size;
                                *ky = rint(gdsarr[2]);
                            } else if (strcmp(var.name, "lon") == 0 || strcmp(var.name, "longitude") == 0 ) {
                                gdsarr[4] = var_val[0];
                                gdsarr[6] = var_val[array_size-1];
                                gdsarr[1] = (float)array_size;
                                *kx = rint(gdsarr[1]);
                            }
                            free(var_val);
                        } else if (var.type == NC_DOUBLE) {
                            //printf("**** car.type == NC_DOUBLE \n");
                            int k;
                            int array_size = 1;
                            for (k = 0; k < var.ndims; k++) {
                                array_size = array_size * dims[var.dims[k]].size;
                            }
                            //printf(" Array size = %i \n", array_size);
                            var_val_double = (double *) malloc(array_size * sizeof(double));
                            nc_status = nc_get_var_double(ncid, varid, var_val_double);

                            if (strcmp(var.name, "lat") == 0 || strcmp(var.name, "latitude") == 0 ) {
                                gdsarr[3] = (float)var_val_double[0];
                                
                                gdsarr[5] = (float)var_val_double[array_size-1];

                                if (gdsarr[3] > gdsarr[5]) {
                                    gdsarr[5] = (float)var_val_double[0];
                                    gdsarr[3] = (float)var_val_double[array_size-1];
                                    *ncfiletype = 3;
                                }
			        gdsarr[2] = (float)array_size;
                                *ky = rint(gdsarr[2]);
                            } else if (strcmp(var.name, "lon") == 0 || strcmp(var.name, "longitude") == 0 ) {
                                gdsarr[4] = (float)var_val_double[0];
				if (gdsarr[4] > 180 ) {
				    gdsarr[4] = gdsarr[4] - 360;
				}
                                gdsarr[6] = (float)var_val_double[array_size-1];
                                if (gdsarr[6] > 180 ) {
                                    gdsarr[6] = gdsarr[6] - 360;
                                }
                                gdsarr[1] = (float)array_size;
                                *kx = rint(gdsarr[1]);
                            }
				//printf("**** gdsarr[3]=%f, gdsarr[4]=%f, gdsarr[5]=%f, gdsarr[6] = %f \n", gdsarr[3], gdsarr[4], gdsarr[5], gdsarr[6]);
                            free(var_val_double);
                        }

                    }
                    free(var.dims);
                }
                gdsarr[7] = 0;
                gdsarr[8] = 0;
                gdsarr[9] = 0;
	    } else if (latlongrid == -1) {
		int havegridnav = 0;
		*ncfiletype = 1;
		/* Search variable attributes for a grid information  (GFE grid) */
		/* 
		 * Grid will have two different grid difinitions
		 * 	1) the grid used to define the GFE domain (Normal lat/lon corner
		 *		proj, angles)
		 *	2) the GFE domain (defines in AWIPS world Coordinates)
		 */
		for (varid = 0; varid < nvars; varid++ ) {
                    nc_status = nc_inq_varndims(ncid, varid, &var.ndims);
                    var.dims = (int *) malloc((var.ndims + 1) * sizeof(int));
                    nc_status = nc_inq_var (ncid, varid, var.name, &var.type, 0, var.dims, &var.natts);
		    /* Find grid information any variable attributes*/
                    if (var.type = NC_FLOAT) {
                        // get variable dimensions
                        int attnum;
			ncatt_t att;
			for (attnum = 0; attnum < var.natts; attnum++) {
			    nc_status = nc_inq_attname (ncid, varid, attnum, att.name);
			    nc_status = nc_inq_att (ncid, varid, att.name, &att.type, &att.len);
			    // Get Attribute values
			    if (att.type == NC_CHAR) {
                        	/* Find Grid Projection information */
                        	att.string = (char *) malloc(att.len+1);
                        	nc_status = nc_get_att_text (ncid, varid, att.name, att.string);
                        	att.string[att.len] = '\0';
				if (strcmp(att.string, "STEREOGRAPHIC") == 0 || strcmp(att.string, "POLAR_STEREOGRAPHIC") == 0) {
                    		    strcpy(cproj, "STR");
                                    gdsarr[0] = (float) 5.0;
				    havegridnav = 1;
               			} else if (strcmp(att.string, "LAMBERT_CONFORMAL") == 0 ) {
                    		    strcpy(cproj, "LCC");
                    		    gdsarr[0] = (float) 3.0;
				    havegridnav = 1;
                		} else if (strcmp(att.string, "CYLINDRICAL_EQUIDISTANT") == 0 ) {
                    		    strcpy(cproj, "CED");
                    		    gdsarr[0] = (float) 0.0;
				    havegridnav = 1;
                		} else if (strcmp(att.string, "MERCATOR") == 0 ) {
                                    strcpy(cproj, "MER");
                                    gdsarr[0] = (float) 1.0;
                                    havegridnav = 1;
				}
				free(att.string);
                    	    } else if (att.type <= NC_INT) {
                       	 	att.ivals = (int *) malloc(att.len * sizeof(int));
                        	nc_status = nc_get_att_int (ncid, varid, att.name, att.ivals);
				if ( strcmp(att.name,"gridSize") == 0) {
				    /* get dmoain grid x,y */
				    gdsarr[1] = (float)att.ivals[0];
                                    gdsarr[2] = (float)att.ivals[1];
				    *kx = rint ( gdsarr [1] );
        			    *ky = rint ( gdsarr [2] );
				} else if (strcmp(att.name,"gridPointUR") == 0) {
				    /* get defining grid x,y */
				    ix = att.ivals[0];
				    iy = att.ivals[1];
				}
				free(att.ivals);
                    	    } else if (att.type == NC_SHORT) {
                        	att.svals  = (short *) malloc(att.len * sizeof(short));
                        	nc_status = nc_get_att_short (ncid, varid, att.name, att.svals);
				free(att.svals);
                    	    } else if (att.type == NC_FLOAT) {
                        	att.fvals = (float *) malloc(att.len * sizeof(float));
                        	nc_status = nc_get_att_float (ncid, varid, att.name, att.fvals);
				/* 
				 * Find defining grid conrners and std parallel angles
				 * and the domain grid world coordinates
				 */
				if ( strcmp(att.name,"latLonLL") == 0) {
				    gdsarr[4] = att.fvals[0];
				    gdsarr[3] = att.fvals[1];
				} else if (strcmp(att.name,"latLonUR") == 0) {
				    gdsarr[6] = att.fvals[0];
                                    gdsarr[5] = att.fvals[1];
				} else if (strcmp(att.name,"latLonOrigin") == 0) {
				    gdsarr[8] = att.fvals[0];
				} else if (strcmp(att.name,"lonOrigin") == 0) {
                                    gdsarr[7] = 90.0;
                                    gdsarr[8] = att.fvals[0];
                                    gdsarr[9] = 0.0;
				} else if (strcmp(att.name,"stdParallelOne") == 0) { 
                                    gdsarr[7] = att.fvals[0];
				} else if (strcmp(att.name,"stdParallelTwo") == 0) {
                                    gdsarr[9] = att.fvals[0];
				} else if (strcmp(att.name,"domainOrigin") == 0) {
				    domainOrig[0] = att.fvals[0];
				    domainOrig[1] = att.fvals[1];
				} else if (strcmp(att.name,"domainExtent") == 0) {
				    domainExt[0] = att.fvals[0];
				    domainExt[1] = att.fvals[1];
				} else if (gdsarr[0] == 1.0 && 
					   strcmp(att.name,"lonCenter") == 0) {
                                    gdsarr[8] = (float)att.dvals[0];
                                    gdsarr[7] = 0.0;
                                    gdsarr[9] = 0.0;
                                }
				free(att.fvals);
                    	    } else if (att.type <= NC_DOUBLE) {
                        	att.dvals = (double *) malloc((att.len+1) * sizeof(double));
                        	nc_status = nc_get_att_double (ncid, varid, att.name, att.dvals);
				/* 
                                 * Find defining grid conrners and std parallel angles
                                 * and the domain grid world coordinates
                                 */
				if ( strcmp(att.name,"latLonLL") == 0) {
                                    gdsarr[4] = (float)att.dvals[0];
				    if (gdsarr[4] < -180) gdsarr[4] = 360 + gdsarr[4];
                                    gdsarr[3] = (float)att.dvals[1];
                                } else if (strcmp(att.name,"latLonUR") == 0) {
                                    gdsarr[6] = (float)att.dvals[0];
				    if (gdsarr[6] < -180) gdsarr[4] = 360 + gdsarr[6];
                                    gdsarr[5] = (float)att.dvals[1];
                                } else if (strcmp(att.name,"latLonOrigin") == 0) {
                                    gdsarr[8] = (float)att.dvals[0];
				} else if (strcmp(att.name,"lonOrigin") == 0) {
                                    gdsarr[7] = 90.0;
                                    gdsarr[8] = att.dvals[0];
                                    gdsarr[9] = 0.0;
                                } else if (strcmp(att.name,"stdParallelOne") == 0) { 
                                    gdsarr[7] = (float)att.dvals[0];
                                } else if (strcmp(att.name,"stdParallelTwo") == 0) {
                                    gdsarr[9] = (float)att.dvals[0];
                                } else if (strcmp(att.name,"domainOrigin") == 0) {
                                    domainOrig[0] = (float)att.dvals[0];
				    domainOrig[1] = (float)att.dvals[1];
                                } else if (strcmp(att.name,"domainExtent") == 0) {
				    domainExt[0] = (float)att.dvals[0];
                                    domainExt[1] = (float)att.dvals[1];
                                } else if (strcmp(att.name,"lonCenter") == 0) {
				    gdsarr[8] = (float)att.dvals[0];
				    gdsarr[7] = 0.0;
				    gdsarr[9] = 0.0;
				}

				free(att.dvals);
                    	    }
			}
		    }
		    if (havegridnav == 1) break;
		}
		/*
		 * Will use grid lat/lon definitons from variable attributes
		 * and domain Origin and Extent and ix, iy
		 */
		gsgprj( cproj, &gdsarr[7], &gdsarr[8], &gdsarr[9], &ix, &iy, &gdsarr[3], &gdsarr[4], &gdsarr[5], &gdsarr[6], iret, strlen(cproj) );

		//printf("iret = %i \n", iret );
		// Convert Domain Origin to grid Lat, lon
		float rlat, rlon;
		int npts = 1;

		gtrans ( sys_G, sys_M, &npts, &domainOrig[0], &domainOrig[1], &rlat, &rlon, iret, strlen(sys_G), strlen(sys_M) );
		gdsarr[3] = rlat;
		gdsarr[4] = rlon;
		float URx = domainOrig[0]+domainExt[0];
		float URy = domainOrig[1]+domainExt[1];

		gtrans ( sys_G, sys_M, &npts, &URx, &URy, &rlat, &rlon, iret, strlen(sys_G), strlen(sys_M) );
		gdsarr[5] = rlat;
		gdsarr[6] = rlon;
            } else {
		*iret = -1;
            }
        }

	if (*kx == -1 || *ky == -1) {
	    *iret = -1;
	}
	corners[0] = gdsarr[3];
	corners[1] = gdsarr[4];
	corners[2] = gdsarr[5];
	corners[3] = gdsarr[6];

	printf(" corners %f %f %f %f \n", corners[0], corners[1], corners[2], corners[3]);
	printf(" proj = %f %f %f \n", gdsarr[7], gdsarr[8], gdsarr[9]);
	/*
         *  Set the Navigation block.
         */

        angflg=1;
        gr_vnav  ( cproj, kx, ky, corners, corners+1,
               corners+2, corners+3, gdsarr+7, gdsarr+8,
               gdsarr+9, &angflg, &valid, &ier, strlen(cproj) );

        if ( ier == 0 )
            gr_mnav  ( cproj, kx, ky, corners, corners+1,
                   corners+2, corners+3, gdsarr+7, gdsarr+8,
                   gdsarr+9, &angflg, navblk, &ier, strlen(cproj) );
        else
            *iret = -37;

}
